// Unit Name: EditColorHandler
// Author:    Frank
// Date:      05-Mai-2006
// Description:
//   ColorHandler fr Eingabeelemente


unit EditColorHandler;

interface
Uses Forms, Classes, Controls, Graphics, StdCtrls, TypInfo;

Const
  // Farbe bei Focus
  _ActiveColor = $009BF5FD;
  _Color = 'Color';

type
  TEditColorHandler = class(TComponent)
    procedure ActiveControlChange(Sender: TObject);
  private
    FActive: Boolean;
    FActiveControl: TWinControl;
    FDefaultColor: TColor;
    FFocusColor: TColor;
    FHandledControls: TList;
    function GetControlColor(AControl: TWinControl): TColor;
    function HandleControlColor(AControl: TWinControl): Boolean;
    procedure SetActive(const Value: Boolean);
    procedure SetControlColor(AControl: TWinControl; AColor: TColor);
    procedure SetFocusColor(const Value: TColor);
    procedure Update;
  protected
    procedure Notification(AComponent : TComponent; Operation : TOperation);
        override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property HandledControls: TList read FHandledControls write FHandledControls;
  published
    property Active: Boolean read FActive write SetActive;
    property FocusColor: TColor read FFocusColor write SetFocusColor default
        _ActiveColor;
  end;

function AppEditColorHandler: TEditColorHandler;

implementation

Var
  FAppEditColorHandler: TEditColorHandler = nil;

function AppEditColorHandler: TEditColorHandler;
begin
  if not Assigned(FAppEditColorHandler) then
    FAppEditColorHandler := TEditColorHandler.Create(nil);
  Result := FAppEditColorHandler;
end;

constructor TEditColorHandler.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFocusColor := _ActiveColor;
  FHandledControls := TList.Create;
end;

destructor TEditColorHandler.Destroy;
begin
  FHandledControls.Free;
  inherited Destroy;
end;

{ ------------------------------------------------------------------------------
 Procedure: ActiveControlChange
 Author:    Frank
 Date:      05-Mai-2006
 Description:
   Screen.onActiveControlChange fr Wechsel der Controls
 See Also:
   SetActive
 Arguments:
   Sender: TObject
 Return Value:
   None
 Summary:
------------------------------------------------------------------------------ }
procedure TEditColorHandler.ActiveControlChange(Sender: TObject);
begin

  if Application.Terminated then
    Exit;


  // Farbe letztes Control zurcksetzen...
  if Assigned(FActiveControl) then
  begin
    SetControlColor(FActiveControl, FDefaultColor);
    FActiveControl.RemoveFreeNotification(Self);
  end;

  if HandleControlColor(Screen.ActiveControl) then
  begin
    FActiveControl := Screen.ActiveControl;
    FDefaultColor  := GetControlColor(FActiveControl);
    SetControlColor(FActiveControl, FFocusColor);
    FActiveControl.FreeNotification(Self);
  end;
end;

{ ------------------------------------------------------------------------------
 Procedure: HandleControlColor
 Author:    Frank
 Date:      05-Mai-2006
 Description:
   Hier wird festgelegt, welche Control gehandelt werden.
 Arguments:
   AControl: TControl
 Return Value:
   None
------------------------------------------------------------------------------ }
function TEditColorHandler.HandleControlColor(AControl: TWinControl): Boolean;
Var
  i : Integer;
begin
  Result := False;
  if not (Assigned(AControl) and AControl.HandleAllocated) then
    exit;

  for i := 0 to FHandledControls.Count - 1 do
    if (AControl is TClass(FHandledControls.Items[i])) then
    begin
      Result := True;
      Break;
    end;
end;

procedure TEditColorHandler.Notification(AComponent : TComponent; Operation :
    TOperation);
begin
  inherited Notification( AComponent, Operation);
  if Assigned(AComponent) then
    if (Operation = opRemove) and (AComponent = FActiveControl) then
      FActiveControl := nil;
end;

procedure TEditColorHandler.SetActive(const Value: Boolean);
begin
  if FActive <> Value then
  begin
    FActive := Value;
    if FActive then
      Screen.OnActiveControlChange := ActiveControlChange
    else
      Screen.OnActiveControlChange := nil;
  end;
end;


// ber RTTI

procedure TEditColorHandler.SetControlColor(AControl: TWinControl; AColor:
    TColor);
Const
  _Color = 'Color';
begin
  if IsPublishedProp(AControl, _Color) then
    SetOrdProp(AControl, _Color, Integer(AColor));
end;

function TEditColorHandler.GetControlColor(AControl: TWinControl): TColor;
begin
  if IsPublishedProp(AControl, _Color) then
    Result := TColor(GetOrdProp(AControl, _Color));
end;


procedure TEditColorHandler.SetFocusColor(const Value: TColor);
begin
  if FFocusColor <> Value then
  begin
    FFocusColor := Value;
    Update;
  end;
end;

{ ------------------------------------------------------------------------------
 Procedure: Update
 Author:    Frank
 Date:      05-Mai-2006
 Description:
   Control neu zeichnen, falls Farbe gendert etc.
 See Also:

 Arguments:
   None
 Return Value:
   None
 Summary:
------------------------------------------------------------------------------ }
procedure TEditColorHandler.Update;
begin
  if Assigned(FActiveControl) then
    if FActive then
      SetControlColor(FActiveControl, FocusColor)
    else
      SetControlColor(FActiveControl, FDefaultColor);
end;

initialization

finalization

  if Assigned(FAppEditColorHandler) then
  begin
    FAppEditColorHandler.Active := False;
    FAppEditColorHandler.Free;
  end;

end.
